;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "clojure-mode-extra-font-locking" "5.23.0"
  "Extra font-locking for Clojure mode."
  '((clojure-mode "5.23.0"))
  :url "https://github.com/clojure-emacs/clojure-mode"
  :commit "9146525680b29a0c6b72a1126d075b54f8066710"
  :revdesc "9146525680b2"
  :keywords '("languages" "lisp")
  :authors '(("Bozhidar Batsov" . "bozhidar@batsov.dev"))
  :maintainers '(("Bozhidar Batsov" . "bozhidar@batsov.dev")))
